{ connexionswing, pkgs, phpPackages, config }:
rec {
  app = connexionswing.override { inherit (config) environment; };
  keys = [{
    dest = "webapps/${app.environment}-connexionswing";
    user = apache.user;
    group = apache.group;
    permissions = "0400";
    text = ''
      # This file is auto-generated during the composer install
      parameters:
          database_host: ${config.mysql.host}
          database_port: ${config.mysql.port}
          database_name: ${config.mysql.name}
          database_user: ${config.mysql.user}
          database_password: ${config.mysql.password}
          database_server_version: ${pkgs.mariadb.mysqlVersion}
          mailer_transport: sendmail
          mailer_host: null
          mailer_user: null
          mailer_password: null
          subscription_email: ${config.email}
          allow_robots: true
          secret: ${config.secret}
      ${if app.environment == "prod" then ''
      services:
        swiftmailer.mailer.default.transport:
            class:     Swift_SendmailTransport
            arguments: ['/run/wrappers/bin/sendmail -bs']
      '' else ""}
    '';
  }];
  phpFpm = rec {
    preStart = ''
      if [ ! -f "${app.varDir}/currentWebappDir" -o \
          ! -f "${app.varDir}/currentKey" -o \
          "${app}" != "$(cat ${app.varDir}/currentWebappDir 2>/dev/null)" ] \
          || ! sha512sum -c --status ${app.varDir}/currentKey; then
        pushd ${app} > /dev/null
        /run/wrappers/bin/sudo -u wwwrun ./bin/console --env=${app.environment} cache:clear --no-warmup
        popd > /dev/null
        echo -n "${app}" > ${app.varDir}/currentWebappDir
        sha512sum /var/secrets/webapps/${app.environment}-connexionswing > ${app.varDir}/currentKey
      fi
      '';
    serviceDeps = [ "mysql.service" ];
    socket = "/var/run/phpfpm/connexionswing-${app.environment}.sock";
    phpConfig = ''
      extension=${phpPackages.imagick}/lib/php/extensions/imagick.so
      '';
    pool = ''
      listen = ${socket}
      user = ${apache.user}
      group = ${apache.group}
      listen.owner = ${apache.user}
      listen.group = ${apache.group}
      php_admin_value[upload_max_filesize] = 20M
      php_admin_value[post_max_size] = 20M
      ;php_admin_flag[log_errors] = on
      php_admin_value[open_basedir] = "/run/wrappers/bin/sendmail:/var/secrets/webapps/${app.environment}-connexionswing:${app}:${app.varDir}:/tmp"
      php_admin_value[session.save_path] = "${app.varDir}/phpSessions"
      ${if app.environment == "dev" then ''
      pm = ondemand
      pm.max_children = 5
      pm.process_idle_timeout = 60
      env[SYMFONY_DEBUG_MODE] = "yes"
      '' else ''
      pm = dynamic
      pm.max_children = 20
      pm.start_servers = 2
      pm.min_spare_servers = 1
      pm.max_spare_servers = 3
      ''}'';
  };
  apache = rec {
    user = "wwwrun";
    group = "wwwrun";
    modules = [ "proxy_fcgi" ];
    webappName = "connexionswing_${app.environment}";
    root = "/run/current-system/webapps/${webappName}";
    vhostConf = ''
    <FilesMatch "\.php$">
      SetHandler "proxy:unix:${phpFpm.socket}|fcgi://localhost"
    </FilesMatch>

    <Directory ${app.varDir}/medias>
      Options FollowSymLinks
      AllowOverride None
      Require all granted
    </Directory>

    <Directory ${app.varDir}/uploads>
      Options FollowSymLinks
      AllowOverride None
      Require all granted
    </Directory>

    ${if app.environment == "dev" then ''
    <Location />
      Use LDAPConnect
      Require ldap-group   cn=connexionswing.immae.eu,cn=httpd,ou=services,dc=immae,dc=eu
      ErrorDocument 401 "<html><meta http-equiv=\"refresh\" content=\"0;url=https://connexionswing.com\"></html>"
    </Location>

    <Directory ${root}>
      Options Indexes FollowSymLinks MultiViews Includes
      AllowOverride None
      Require all granted

      DirectoryIndex app_dev.php

      <IfModule mod_negotiation.c>
      Options -MultiViews
      </IfModule>

      <IfModule mod_rewrite.c>
        RewriteEngine On

        RewriteCond %{REQUEST_URI}::$1 ^(/.+)/(.*)::\2$
        RewriteRule ^(.*) - [E=BASE:%1]

        # Maintenance script
        RewriteCond %{DOCUMENT_ROOT}/maintenance.php -f
        RewriteCond %{SCRIPT_FILENAME} !maintenance.php
        RewriteRule ^.*$ %{ENV:BASE}/maintenance.php [R=503,L]
        ErrorDocument 503 /maintenance.php

        # Sets the HTTP_AUTHORIZATION header removed by Apache
        RewriteCond %{HTTP:Authorization} .
        RewriteRule ^ - [E=HTTP_AUTHORIZATION:%{HTTP:Authorization}]

        RewriteCond %{ENV:REDIRECT_STATUS} ^$
        RewriteRule ^app_dev\.php(?:/(.*)|$) %{ENV:BASE}/$1 [R=301,L]

        # If the requested filename exists, simply serve it.
        # We only want to let Apache serve files and not directories.
        RewriteCond %{REQUEST_FILENAME} -f
        RewriteRule ^ - [L]

        # Rewrite all other queries to the front controller.
        RewriteRule ^ %{ENV:BASE}/app_dev.php [L]
      </IfModule>

    </Directory>
    '' else ''
    Use Stats connexionswing.com

    <Directory ${root}>
      Options Indexes FollowSymLinks MultiViews Includes
      AllowOverride All
      Require all granted
    </Directory>
    ''}
    '';
  };
  activationScript = {
    deps = [ "wrappers" ];
    text = ''
    install -m 0755 -o ${apache.user} -g ${apache.group} -d ${app.varDir} \
      ${app.varDir}/medias \
      ${app.varDir}/uploads \
      ${app.varDir}/var
    install -m 0750 -o ${apache.user} -g ${apache.group} -d ${app.varDir}/phpSessions
    '';
  };
}
